# -*- coding: utf-8 -*-
from isort import SortImports

import os
try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import ConfigParser


class Flake8Isort(object):
    name = 'flake8_isort'
    version = '0.1'
    message = 'I001 found unsorted imports'

    config_file = None

    def __init__(self, tree, filename):
        self.filename = filename

    @classmethod
    def add_options(cls, parser):
        parser.add_option(
            '--no-isort-config',
            action='store_true',
            help='Do not require an .isort.cfg file to be found'
        )
        parser.config_options.append('no-isort-config')

    @classmethod
    def parse_options(cls, options):
        if options.no_isort_config is None:
            cls.config_file = True
        else:
            cls.config_file = False

    def run(self):
        if self.config_file and not self.search_isort_config():
            yield 0, 0, 'I002 no .isort.cfg file found', type(self)
        else:
            sort_result = SortImports(self.filename, check=True)
            if sort_result.incorrectly_sorted:
                yield 0, 0, self.message, type(self)

    def search_isort_config(self):
        """Search for a .isort.cfg or setup.cfg all the way up to the root
        folder
        """
        full_path = os.path.abspath(self.filename)
        path_parts = full_path.split(os.path.sep)
        dirs_missing = len(path_parts)

        while dirs_missing > 0:
            dirs_missing -= 1
            partial_parts = path_parts[:dirs_missing]
            partial_path = os.sep.join(partial_parts)

            isort_file = '{0}{1}.isort.cfg'.format(partial_path, os.sep)
            if os.path.exists(isort_file):
                return True

            # If the setup file exists and has an "isort" section,
            # then we've found the configuration.
            setup_file = os.path.join(partial_path, 'setup.cfg')
            if os.path.exists(setup_file):
                config = ConfigParser()
                config.read(setup_file)
                if 'isort' in config.sections():
                    return True

        return False



# -*- coding: utf-8 -*-
from flake8.engine import get_style_guide
from flake8_isort import Flake8Isort
from tempfile import mkdtemp
from testfixtures import OutputCapture

import os
import sys
import unittest


class TestFlake8Isort(unittest.TestCase):

    def _given_a_file_in_test_dir(self, contents, isort_config):
        test_dir = os.path.realpath(mkdtemp())
        file_path = os.path.join(test_dir, 'test.py')
        isort_path = os.path.join(test_dir, '.isort.cfg')
        with open(file_path, 'w') as a_file:
            a_file.write(contents)

        with open(isort_path, 'w') as a_file:
            a_file.write('[settings]')
            a_file.write(isort_config)

        return file_path

    def test_sorted_correctly(self):
        file_path = self._given_a_file_in_test_dir(
            'from sys import path\n'
            '\n'
            'import os'
            '\n',
            isort_config='force_single_line=True\nforce_alphabetical_sort=True'
        )
        with OutputCapture():
            checker = Flake8Isort(None, file_path)
            ret = list(checker.run())
            self.assertEqual(ret, [])

    def test_sorted_incorrectly(self):
        file_path = self._given_a_file_in_test_dir(
            'from sys import pid\n'
            'import threading',
            isort_config='force_single_line=True'
        )
        with OutputCapture():
            checker = Flake8Isort(None, file_path)
            ret = list(checker.run())
            self.assertEqual(len(ret), 1)
            self.assertEqual(ret[0][0], 0)
            self.assertEqual(ret[0][1], 0)
            self.assertEqual(ret[0][2], 'I001 found unsorted imports')

    def test_isortcfg_found(self):
        # _given_a_file_in_test_dir already creates an .isort.cfg file
        file_path = self._given_a_file_in_test_dir(
            'from sys import pid\n'
            'import threading',
            isort_config='force_single_line=True'
        )
        with OutputCapture():
            checker = Flake8Isort(None, file_path)
            checker.config_file = True
            ret = list(checker.run())
            self.assertEqual(len(ret), 1)
            self.assertEqual(ret[0][0], 0)
            self.assertEqual(ret[0][1], 0)
            self.assertEqual(ret[0][2], 'I001 found unsorted imports')

    def test_isortcfg_not_found(self):
        file_path = self._given_a_file_in_test_dir(
            'from sys import pid\n'
            'import threading',
            isort_config='force_single_line=True'
        )
        # remove the .isort.cfg file
        isortcfg_path = file_path.split('/')[: -1]
        isortcfg_path = '{0}/.isort.cfg'.format('/'.join(isortcfg_path))
        os.remove(isortcfg_path)

        with OutputCapture():
            checker = Flake8Isort(None, file_path)
            checker.config_file = True
            ret = list(checker.run())
            self.assertEqual(len(ret), 1)
            self.assertEqual(ret[0][0], 0)
            self.assertEqual(ret[0][1], 0)
            self.assertEqual(ret[0][2], 'I002 no .isort.cfg file found')

    def test_default_option(self):
        """By default a config file (.isort.cfg) is expected"""
        _argv = sys.argv
        try:
            sys.argv = []
            get_style_guide(parse_argv=True)  # parse arguments
            self.assertTrue(Flake8Isort.config_file)
        finally:
            sys.argv = _argv

    def test_config_file(self):
        """Check that one can force to not look for a config file"""
        _argv = sys.argv
        try:
            sys.argv = ['', '--no-isort-config']
            get_style_guide(parse_argv=True)  # parse arguments
            self.assertFalse(Flake8Isort.config_file)
        finally:
            sys.argv = _argv


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from setuptools import setup


short_description = 'flake8 plugin that integrates isort .'

long_description = '{0}\n{1}'.format(
    open('README.rst').read(),
    open('CHANGES.rst').read()
)

setup(
    name='flake8-isort',
    version='1.1',
    description=short_description,
    long_description=long_description,
    # Get more from http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 3 - Alpha',
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Operating System :: OS Independent",
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development',
    ],
    keywords='pep8 flake8 isort imports',
    author='Gil Forcada',
    author_email='gil.gnome@gmail.com',
    url='https://github.com/gforcada/flake8-isort',
    license='GPL version 2',
    py_modules=['flake8_isort', ],
    include_package_data=True,
    test_suite = 'run_tests',
    zip_safe=False,
    install_requires=[
        'flake8',
        'isort',
    ],
    extras_require={
        'test': [
            'testfixtures',
        ],
    },
    entry_points={
        'flake8.extension': ['I00 = flake8_isort:Flake8Isort'],
    },
)
